The `@Behaviour` keyword declares the behaviour used for the
test. This keyword must be followed by an option specifying the
interface used by the behaviour. The following interfaces are
supported for `MFront` generated behaviours:

- `castem` (`umat`)
- `cyrano`
- `aster`
- `europlexus` (`epx`)
- `abaqus` (`abaqus_standard` or `abaqus_umat`)
- `abaqus_explicit` (`abaqus_vumat`)

Some interfaces are also available for behaviours not generated by
`MFront`:

- `mistral`
- `castem_umat_small_strain`
- `castem_umat_finite_strain`

Two strings are then expected:

- the library in which the behaviour is implemented;
- the name of the function.

Sometimes, additional parameters must be given, in particular for
interfaces to behaviours not generated by `MFront`.

## Example

~~~~ {.cpp}
@Behaviour<castem> 'libMFrontCastemBehaviours.so' 'umatnorton';
~~~~~~~~

~~~~ {.cpp}
@Behaviour<mistral> 'SOURCES/libumat.so' 'umat_' {
 'mistral_parameters_file' : 'M5CC_par.txt',
 'SENSIP1':1,
 'SENSIP2':3,
 'ICBASE' :0};
~~~~~~~~

~~~~ {.cpp}
@Behaviour<castem_umat_small_strain> './libumat.so' 'umat_'{
 'NUME_LOI'   : 10,
 'SYMETRIE'   : 'ISOTROPE',
 'C_MATERIAU' : {'YOUNG','NU'}
};
~~~~~~~~

